function symbol = prcsymb(data, data_carriers, PRCdata, PRC_carriers, bitsHz_data, bitsHz_PRC,...
	Data_QAM_PSK, PRC_QAM_PSK, ifft_size, format)
%PRCSYMB Generates a DMT (OFDM) symbol with data and Peak reduction carriers
%symbol = prcsymb(data, data_carriers, PRCdata, PRC_carriers, bitsHz_data, bitsHz_PRC,
%	Data_QAM_PSK, PRC_QAM_PSK, ifft_size, format)
%
%data : contains a vector of the data to transmit in integer
%	i.e. for BPSK: [0,1,1,0,1,0,0,0,1....]
%	QPSK: [3,1,2,0,2,1,2,3,0....]
%	16QAM: [15,2,5,1,0,14 ....]
%	Note: The length of data should match the number of carriers.
%	If not a warning message will be generated, and they will be
%	zero padded.
%	If the data is given as -1 then that carrier is no transmitted on
%carriers : This is a list of the carriers to use.
%	PRC_carriers should be different to data_carriers
%bitsHz : Sets the bits/Hz modulation. i.e. 2 - QPSK, 4 - 16QAM, 16PSK
%QAM_PSK : Set whether to use PSK or QAM modulation
%	QAM_PSK : 0 - PSK, 1 - QAM
%ifft_size : size of the ifft used to generate the symbol.
%	The default ifft_size will be calculated based on the
%	size of the data given, the number of carriers, and the format
%format : Determines whether a real or complex result is generated
%	format : 'real', A real waveform is generated. The positive and
%	negative half spectrums are set to be conjugates of one another
%	Only the positive carriers can be set (Lower half of the spectrum).
%	i.e. valid carriers are	from 1 : ifft_size/2
%	format : 'complex', A complex waveform is generated, allowing all
%	carriers to be set. Note : the default is 'real'
%	NOTE: I have only implemented upto 8QAM, any PSK can be handled

if (~strcmp(format,'complex'))
	format = 'real';
end

MaxModData = 2^bitsHz_data-1;
MaxModPRC = 2^bitsHz_PRC-1;
DataLen = length(data);
CarrLen = length(data_carriers);
DataLenPRC = length(PRCdata);
CarrLenPRC = length(PRC_carriers);
debug = 0;
if (debug == 1),
if (DataLen > CarrLen)
	error('Too much data to fit into one data symbol');
end
if (DataLen < CarrLen)
	disp('WARNING: Data is shorter then number of carriers, it will be zero padded');
	data = [data zeros(1,CarrLen-DataLen)];
end
if (DataLenPRC > CarrLenPRC)
	error('Too much peak reduction data to fit into one data symbol in the allocated PRC');
end
if (DataLen < CarrLen)
	error('PRC data is shorter then number of PRC carriers');
end

maxcarr = max([max(data_carriers) max(PRC_carriers)]);
if (((maxcarr > ifft_size/2)&strcmp(format,'real'))|...
	(maxcarr > ifft_size))
	error('Carriers (data or PRC) too high to fit in IFFT')
end

%Check if the modulation scheme is legal
%(Only check the first 50 to reduce computational overhead)
if (max(data(1:min(50:DataLen))) > MaxModData)
	error('Data has input which is too big for specified modulation scheme');
end

end
if (length(find(data == -1)) > 0)
	%remove carriers with data of -1
	index = find(data ~= -1);
	data_carriers = data_carriers(index);
	data = data(index);
end;

if (length(find(PRCdata == -1)) > 0)
	%remove PR carriers with data of -1
	index = find(PRCdata ~= -1);
	PRC_carriers = PRC_carriers(index);
	PRCdata = PRCdata(index);
end;

%Convert input data into phase and amplitude based on the modulation scheme,
%ready to set the spectrum
%Generate lookup tables to convert from the data to the Complex spectral components
if (Data_QAM_PSK == 1),
	%QAM
	if (bitsHz_data == 1),
		%BPSK
		ModMag = [ 1 1 ];
		ModPh = [ 0 pi ];
	elseif (bitsHz_data == 2),
		%QPSK
		ModMag = ones(1,4);
		ModPh = [0 pi/2 pi 3*pi/2];
	elseif (bitsHz_data == 3),
		%8QAM
		ModMag = [ 0.5*ones(1,4) ones(1,4)];
		%ModPh = [ pi/4:pi/2:2*pi, 0:pi/2:3*pi/2];
		ModPh = [ 0:pi/2:3*pi/2, 0:pi/2:3*pi/2];
	elseif (bitsHz_data == 4),
		%2 bits of amplitude and 2 bits of phase
		ModMag = [0.25*ones(1,4) 0.5*ones(1,4) 0.75*ones(1,4) ones(1,4)];
		ModPh = [0:pi/2:3*pi/2, 0:pi/2:3*pi/2, 0:pi/2:3*pi/2, 0:pi/2:3*pi/2];
	else
		error('Modulation 32QAM and higher currently unsupported');
	end

elseif (Data_QAM_PSK == 0),
	%PSK
	ModMag = ones(1,2^bitsHz_data);
	ModPh = 0:2*pi/(2^bitsHz_data):2*pi*(1-1/(2^bitsHz_data));
end
%Convert the magnitude and phase into complex numbers
[X,Y] = pol2cart(ModPh,ModMag);
DataModTable = X+sqrt(-1)*Y;

%Convert input PRC data into phase and amplitude based on the modulation scheme,
%ready to set the spectrum
%Generate lookup tables to convert from the data to the Complex spectral components
if (PRC_QAM_PSK == 1),
	%QAM
	if (bitsHz_PRC == 1),
		%BPSK
		ModMag = [ 1 1 ];
		ModPh = [ 0 pi ];
	elseif (bitsHz_PRC == 2),
		%QPSK
		ModMag = ones(1,4);
		ModPh = [0 pi/2 pi 3*pi/2];
	elseif (bitsHz_PRC == 3),
		%8QAM
		ModMag = [ 0.5*ones(1,4) ones(1,4)];
		%ModPh = [ pi/4:pi/2:2*pi, 0:pi/2:3*pi/2];
		ModPh = [ 0:pi/2:3*pi/2, 0:pi/2:3*pi/2];
	elseif (bitsHz_PRC == 4),
		%2 bits of amplitude and 2 bits of phase
		ModMag = [0.25*ones(1,4) 0.5*ones(1,4) 0.75*ones(1,4) ones(1,4)];
		ModPh = [0:pi/2:3*pi/2, 0:pi/2:3*pi/2, 0:pi/2:3*pi/2, 0:pi/2:3*pi/2];
	elseif (bitsHz_PRC == 5),
		%2 bits of amplitude and 3 bits of phase
		ModMag = [0.25*ones(1,8) 0.5*ones(1,8) 0.75*ones(1,8) ones(1,8)];
		ModPh = [0:pi/4:7*pi/4, 0:pi/4:7*pi/4, 0:pi/4:7*pi/4, 0:pi/4:7*pi/4];
	else
		error('Modulation 32QAM and higher currently unsupported');
	end

elseif (PRC_QAM_PSK == 2)
	%QAM
	if (bitsHz_PRC == 1),
		%BPSK
		ModMag = [ 1 1 ];
		ModPh = [ 0 pi ];
	elseif (bitsHz_PRC == 2),
		%QPSK
		ModMag = ones(1,4);
		ModPh = [0 pi/2 pi 3*pi/2];
	elseif (bitsHz_PRC == 3),
		%8QAM
		ModMag = [ 0.5*ones(1,4) ones(1,4)];
		%ModPh = [ pi/4:pi/2:2*pi, 0:pi/2:3*pi/2];
		ModPh = [ 0:pi/2:3*pi/2, 0:pi/2:3*pi/2];
	elseif (bitsHz_PRC == 4),
		%1 bits of amplitude and 3 bits of phase
		ModMag = [0.5*ones(1,8) ones(1,8)];
		ModPh = [0:pi/4:7*pi/4, 0:pi/4:7*pi/4];
	elseif (bitsHz_PRC == 5),
		%1 bits of amplitude and 4 bits of phase
		ModMag = [0.5*ones(1,16) ones(1,16)];
		ModPh = [0:pi/8:15*pi/8, 0:pi/8:15*pi/8];
	else
		error('Modulation 32QAM and higher currently unsupported');
	end

elseif (PRC_QAM_PSK == 0),
	%PSK
	ModMag = ones(1,2^bitsHz_PRC);
	ModPh = 0:2*pi/(2^bitsHz_PRC):2*pi*(1-1/(2^bitsHz_PRC));
end
%Convert the magnitude and phase into complex numbers
[X,Y] = pol2cart(ModPh,ModMag);
PRCModTable = X+sqrt(-1)*Y;

%=============================================
% Use the modulation lookup table to find the
% spectral components for the data
%=============================================

%This is a work around for a bug in Matlab 4.2
%It does not dereference  a table correctly when the index vector 
%is all ones;
%if (data == zeros(size(data))),
%	DataModData = ones(size(data)).*DataModTable(1);
%else
	DataModData = DataModTable(data+1);
%end

%if (PRCdata == zeros(size(PRCdata))),
%	PRCModData = ones(size(PRCdata)).*PRCModTable(1);
%else
	PRCModData = PRCModTable(PRCdata+1);
%end


CombCarriers = [data_carriers PRC_carriers];
CombModData = [DataModData PRCModData];

if (strcmp(format,'real')),
	%Now assign the data into carriers
	Spect = zeros(1,ifft_size/2);	%Start with a blank spectrum which
					%is half the size of the ifft
	Spect(CombCarriers) = CombModData;	%Put the data is the carriers from
					%lowest to highest
	%Copy the conjugate from the lower half into the upper half to
	%make it a real spectrum
	Spect = [ 0 Spect(2:ifft_size/2) 0 conj(Spect(ifft_size/2:-1:2))];
else
	Spect = zeros(1,ifft_size);
	Spect(CombCarriers) = CombModData;
end

%Take the IFFT to convert the spectrum back
symbol = ifft(Spect);

%Remove very small complex components from the real signal
if (strcmp(format,'real'))
	symbol = real(symbol);
end
