function objs = genmodel(modeltype)
%GENMODEL This function returns an object model from a set of premade models
% genmodel(modeltype)
% modeltype = 1, Electrical Engineering at James Cook University 2nd Floor
% modeltype = 2, Two ECE building near each other, with an offset of 10+30i
% modeltype = 3, Empty model, can be used to see the propagation effect with no objects
% modeltype = 4, Outer Walls of the two ECE buildings in modeltype = 2. Used
% with modeltype = 3 for plotting the boundaries of the buildings. Used in comparison
% between propagation with and without the building present
%========================================================================================
% Each model is a set on objects stored in an array. Each object is a structure with the
% following fields:
% vertices    - complex coordinate of the vertex points of the object
% reflection  - reflectiveness of the object (fraction of incident power reflected)
% attenuation - power attenuation of a ray passing through the object in dB per metre
% roughness   - roughness of the surface of the object. This is the standard deviation
%	(in radians) of the reflected (and transmitted) ray power. for example
%	roughness of 0.01 is a highly polished surface, a roughness of >10
%	would simulate light reflection off white paper, i.e. the diffuse reflection

%Copyright Eric Lawrey August 2000
%version 1.2;
%3/8/00 Version 1.2
%Added ECEbuilding2 function which adds benchs and other scatter object to effectively 
%increase the pathloss exponent

switch modeltype
case 1
   objs = ECEbuilding;
case 2
   offset = 10+30*i;
   objs = ECEbuilding;
   for k = 1:length(objs)
      objstemp = objs(k);
      objstemp.vertices = objstemp.vertices+offset;
      objs(length(objs)+1) = objstemp;
   end
case 3
   objs = [];
case 4
   offset = 10+30*i;
   objs = addobj(closedroom(52.9,16.6,0.3)-(0.05+i*0.05)+offset,0.95,100,0.02);
   objs(length(objs)+1) = addobj(closedroom(52.9,16.6,0.3)-(0.05+i*0.05),0.95,100,0.02);
case 5
   offset = 10+30*i;
   objs1 = ECEbuilding;
   objs = objs1;
   for k = 1:length(objs1)
      objstemp = objs1(k);
      objstemp.vertices = objstemp.vertices+offset;
      objs(length(objs)+1) = objstemp;
   end
   offset = 0+60*i;
   for k = 1:length(objs1)
      objstemp = objs1(k);
      objstemp.vertices = objstemp.vertices+offset;
      objs(length(objs)+1) = objstemp;
   end
case 6
   objs = ECEbuilding2;
end
%========================================================================================

function obj = closedroom(xsize,ysize,wallthickness)
%makes a closed room shape
w = wallthickness;
xs = xsize;
ys = ysize;
x = [0 0 xs,xs,w,w,xs-w,xs-w,w,w];
y = [ys,0 0 ys ys ys-w,ys-w,w,w,ys];
obj = x+i*y;

%========================================================================================
function objs = addobj(vertices,reflection,attenuation,roughness)
%ADDOBJ returns a structure for an object that can be added to the radio model
% objs = addobj(vertices,reflection,attenuation,roughness)
% Fields:
% vertices    - complex coordinate of the vertex points of the object
% reflection  - reflectiveness of the object (fraction of incident power reflected)
% attenuation - power attenuation of a ray passing through the object in dB per metre
% roughness   - roughness of the surface of the object. This is the standard deviation
%	(in radians) of the reflected (and transmitted) ray power. for example
%	roughness of 0.01 is a highly polished surface, a roughness of >10
%	would simulate light reflection off white paper, i.e. the diffuse reflection
% example:
% objs = addobj(closedroom(50,20,1),0.8,30,0.1);
% objs(2) = addobj(wall(15,0.5,pi/4,5,5),0.4,2,1);
% plotmodelobj(objs)
%
%See also PLOTMODELOBJ

%Copyright Eric Lawrey Jan 2000
objs = struct('vertices',vertices,'reflection',reflection,'attenuation',attenuation,'roughness',roughness);

%========================================================================================
function vertices = wall(length,width,angle,xoff,yoff)
%WALL returns the vertices for a wall object
%vertices = wall(length,width,angle,xoff,yoff)
%The wall has a lower left center of xoff,yoff
%xoff and yoff can left off and the default of
%zero will be used

%Copyright Eric Lawrey, Jan 2000
if nargin < 5
   yoff = 0;
end

if nargin < 4
   xoff = 0;
end
tangent = angle+pi/2;
[xt,yt] = pol2cart(tangent,width);
t = (xt+sqrt(-1)*yt)/2;
[xl,yl] = pol2cart(angle,length);
l = xl+sqrt(-1)*yl;
offset = xoff+sqrt(-1)*yoff;
vertices = [t -t l-t l+t]+offset;

function objs = ECEbuilding
   %Make the engineering building model
   %Plaster board wall
   pwref = 0.1;	%Reflectiveness
   pwatten = 8;	%Attenuation in dB/m
   pwrough = 0.1;	%Roughness in radians 
   pwthick = 0.13;	%Thickness of the wall in m
   %Concrete wall
   cref = 0.15;
   catten = 16;
   crough = 0.1;
   cthick = 0.2;
   %Add outer concrete walls
   objs = addobj(closedroom(52.8,16.5,cthick),cref,catten,crough);
   %Add reflective shielding on the building
   sthick = 0.04;
   objs(length(objs)+1) = addobj(closedroom(52.9,16.6,sthick)-(0.05+i*0.05),0.2,22,0.02);
   
   %Add inner walls
   objs(length(objs)+1) = addobj(wall(12.9-cthick/2,pwthick,0,cthick,9.9),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(52.8-2*cthick,pwthick,0,cthick,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(13.5,cthick,0,12.9+cthick/2,8.7),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(26.4,pwthick,0,26.4,9.9),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(9.9-cthick-pwthick/2,pwthick,pi/2,6.3,cthick),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(9.9-cthick-pwthick/2,cthick,pi/2,12.9,cthick),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(8.7-cthick*1.5,cthick,pi/2,26.4,cthick),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(8.7-cthick*1.5,cthick,pi/2,20,cthick),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(6.4-cthick,cthick,0,20+cthick/2,6),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(1.2-cthick/2+pwthick/2,pwthick,pi/2,26.4,8.7+cthick/2),pwref,pwatten,pwrough);
   
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,10.8,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,18.3,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,26.4,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,39.9,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(9.9,pwthick,pi/2,39.9,0),pwref,pwatten,pwrough);
   
   
   function objs = ECEbuilding2
   %This model has the walls of the engineering build plus internal benches. From simulations with
   %s0010_expcompare It was found that the added objects in ECEbuilding2 made the simulation
   %worse.
   %Make the engineering building model
   %Plaster board wall
   pwref = 0.1;	%Reflectiveness
   pwatten = 8;	%Attenuation in dB/m
   pwrough = 0.1;	%Roughness in radians 
   pwthick = 0.13;	%Thickness of the wall in m
   %Concrete wall
   cref = 0.15;
   catten = 16;
   crough = 0.1;
   cthick = 0.2;
   %Object scatter
   objref = 0.1;
   objatten = 3;
   objrough = 1;
   
   %Add outer concrete walls
   objs = addobj(closedroom(52.8,16.5,cthick),cref,catten,crough);
   %Add reflective shielding on the building
   sthick = 0.04;
   objs(length(objs)+1) = addobj(closedroom(52.9,16.6,sthick)-(0.05+i*0.05),0.2,22,0.02);
   
   %Add inner walls
   objs(length(objs)+1) = addobj(wall(12.9-cthick/2,pwthick,0,cthick,9.9),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(52.8-2*cthick,pwthick,0,cthick,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(13.5,cthick,0,12.9+cthick/2,8.7),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(26.4,pwthick,0,26.4,9.9),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(9.9-cthick-pwthick/2,pwthick,pi/2,6.3,cthick),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(9.9-cthick-pwthick/2,cthick,pi/2,12.9,cthick),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(8.7-cthick*1.5,cthick,pi/2,26.4,cthick),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(8.7-cthick*1.5,cthick,pi/2,20,cthick),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(6.4-cthick,cthick,0,20+cthick/2,6),cref,catten,crough);
   objs(length(objs)+1) = addobj(wall(1.2-cthick/2+pwthick/2,pwthick,pi/2,26.4,8.7+cthick/2),pwref,pwatten,pwrough);
   
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,10.8,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,18.3,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,26.4,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(5.1,pwthick,pi/2,39.9,11.4),pwref,pwatten,pwrough);
   objs(length(objs)+1) = addobj(wall(9.9,pwthick,pi/2,39.9,0),pwref,pwatten,pwrough);
   
   benchw = [1.3 1.3 1.3 2.3 2.3 2.3 2.3 2.1 2.1 1.3 1.3 1.3 1.3 1.3 1.3 1.3 1 1 1 1 1 1 1 1];
   benchh = [5 3.5 3.5 3.5 3.5 3.5 3.5 2.3 2.3 9.6 3 3 3 3 3 3 5 5 5 5 2 2 2 2];
   %Bench scatter
   benchref = 0.01;
   benchatten = 3;
   benchrough = 1;   
   %Uwall = 16.2-benchh();
   Lwall = cthick;
   %Mwall = 9.9-benchh-pwthick/2;

   X = [39.9+pwthick/2 39.9-benchw(2)-pwthick/2 39.9-benchw(3)-pwthick/2 34.96 34.96 30.68 30.68 47.06...
         42.88 51.3 26.4-benchw(11)-pwthick/2 18.3-benchw(12)-pwthick/2 13.5 8.20 5 3 ...
         43 45 47 49 43 45 47 49];
   Y = [Lwall Lwall 9.9-benchh(3)-pwthick/2 Lwall 9.9-benchh(5)-pwthick/2 9.9-benchh(6)-pwthick ...
         Lwall 16.25-benchh(8) 16.25-benchh(9) Lwall 16.25-benchh(11:15) 2.84 2 2 2 2 7.8 7.8 7.8 7.8]+benchh/2;
   for k = 1:length(X)
      objs(length(objs)+1) = addobj(wall(benchw(k),benchh(k),0,X(k),Y(k)),benchref,benchatten,benchrough);
   end
   
   benchw = [5 5 5];
   benchh = [1.3 1.3 1.3];
   %Bench scatter
   benchref = 0.01;
   benchatten = 3;
   benchrough = 1;
   X = [39.9+pwthick/2 28.69 6.3+pwthick/2];
	Y = [11.4+cthick/2 11.4+cthick/2 7.5 ]+benchh/2;
   for k = 1:length(X)
      objs(length(objs)+1) = addobj(wall(benchw(k),benchh(k),0,X(k),Y(k)),benchref,benchatten,benchrough);
   end