function B = kaiser2(N,arg1,arg2)
%KAISER2 window of a given side lobe power, or transition width
% B = kaiser2(N,LobePow);
% This returns the window function corresponding to a kaiser
% window, which will have a first side lobe power of 'LobePow',
% where 'LobePow' is in dBc. Valid values of LobePow are
% -13.3 to -280 dBc.
% This program is most accurate for larger number of taps in
% the window function (>= 64 taps). For low number of taps
% the error can be quite large, particularly for < 10 taps.
% The error in this case will be in the order of 10-20%
% 
% For example:
% B = kaiser2(128,-93), will return a 128 tap window function
% with a side lobe attenuation of 93dB. This is similar to a
% 4 term Blackman Harris window function.
% 
% Another method for choosing the window is to specify the
% required frequency transition from peak to first null.
% The wider the transition the lower the first side lobe power.
% B = kaiser2(N,'width',TransWidth);
% Valid values for the transition width are 1 to 11.2.
% Blackman Harris window function has a transition width of
% about 4.1, while boxcar has a transition width of 1.
%
% Note: Kaiser2 does not check the text for the string 'width', it
% just uses the fact that the second argument is a string to
% indicate that it is to use the transition width.
%
% Kaiser2 uses a precomputed lookup table to find the appropriate
% value of alpha to use. This table is in the form of a .mat file.
% It can be generated by running kaiser2('setup'). This also
% plots the relationships between alpha, side lobe power, and
% transition width. 
%
% The setup saves the lookup table to kaiser2.mat, which should
% be placed in the matlab path, or in the same directory as
% kaiser2.
%
% TransWidth = kaiser2('width',SideLobeAtten);
% This finds the transition width of a window for a given side lobe 
% attenuation. This is useful for calculating the number of taps
% required for a particular filter. For example:
% If we want a filter with a transition width which is 5% of
% the nyquist bandwidth, and has a side lobe attenuation of 40 dB.
% The cutoff frequency frequency is 0.25 of nyquist.
%
% Tf = TransWidth/Ntaps, where Tf is the transition width of the
% filter as a fraction of the nyquist frequency.
% 
% Atten = 40-13;		
% TransWidth = kaiser2('width',-Atten);
% Ntaps = ceil(TransWidth/(0.05/2));		%/2 is to normalise to nyquist BW
% Wind = kaiser2(Ntaps,-Atten);
% B = fir1(Ntaps-1,0.25*(1+2/Ntaps),Wind);
% freqz(B);			%Plot the filter
%
% The attenuation of the filter is greater than window function, by approx
% 14dB. This amount increases with side band attenuation, and is due to 
% the FIR1 function.
% 
% SideLobeAtten = kaiser2('atten',TransWidth);
% This finds the side lobe attenuation for a given transition width
%
% See also BLACKH4, HANNING, HAMMING, BOXCAR, LAWREY5
%
% Copyright Eric Lawrey March 2001


% Version 1.0 17th March 2001
% 14/4/01
% Added the option of using kaiser2 to calculate the transition width
% verses side lobe power and vice versa.

if nargin < 1
   error('At least one input is required for kaiser2');
end

%If the first input is a string, run the setup script
if isstr(N)
   switch lower(N)
   case 'setup'
      
   
   N = 100;			%Number alpha values to simulate
   %If N is made to large the lookup table fails to be
   %monotonic, preventing it from being used as a lookup
   %table. I found that N = 200 fails.
   
   WindLength = [128];		%Window length
   %It turns out that the required value for alpha varies
   %slightly with the length of the window function, mainly
   %for low number of taps. So run the setup for different 
   %lengths.
   %However I found that for small values of window length the
   %output is not monotonic and this no good for reverse lookup
   FreqPoints = 2^15;	%A large number of points is needed to accurately
   %							find the nulls
   %============================================
   %		Kaiser Window function
   %============================================
   alpha = linspace(0,35,N);
   NullFreq = zeros(N,length(WindLength));
   SideLobePow = zeros(N,length(WindLength));
   for w = 1:length(WindLength)
      disp(['Calculating Window length : ' num2str(WindLength(w))]);
      for k = 1:length(alpha)
         
         B2 = kaiser(WindLength(w),alpha(k));   
         fft1 = fft(B2/sum(B2),FreqPoints); 
         f = linspace(0, length(B2)/2,FreqPoints/2); 
         P = 20*log10(abs(fft1(1:end/2))+eps);
         %Find the Nulls and the peaks by looking a the 
         %differential of the frequency response of the window
         %function.
         nind = find(diff(sign(diff(abs(fft1(1:end/2)))))>0)+1;
         pind = find(diff(sign(diff(abs(fft1(1:end/2)))))<0)+1;
         if (isempty(nind)|isempty(pind))
		      NullFreq(k,w) = NaN;
            SideLobePow(k,w) = NaN;
            else
         NullFreq(k,w) = f(nind(1));
         SideLobePow(k,w) = P(pind(1)');
         end
         if rem(k,10)==0
            disp(['   Calculating Alpha: ' num2str(k) ' of ' num2str(N)]);
         end
         
      end
   end
   figure(1)
   plot(alpha,SideLobePow);
   xlabel('Kaiser Alpha Value');
   ylabel('First Side Lobe Power (dB)');
   
   figure(2)
   plot(alpha,NullFreq);
   xlabel('Kaiser Alpha Value');
   ylabel('Freq Transition Width, Bins');
   
   save('kaiser2.mat','WindLength', 'alpha','SideLobePow','NullFreq');
case 'width'
   %=====================================================
   %	Find Transition Width for a given Side Lobe Power
   %=====================================================
   if nargin < 2
      error(['Needs two inputs, requires the side lobe attenuation'])
   end
   if ~exist('kaiser2.mat','file')
      disp('The lookup table for kaiser2 does not exist, run');
      error('kaiser2(''setup'') to build the table');
   end
   load('kaiser2.mat') 
	B = interp1(SideLobePow,NullFreq,arg1,'linear'); 
   
case 'atten'
   if nargin < 2
      error(['Needs two inputs, requires the window width'])
   end
   if ~exist('kaiser2.mat','file')
      disp('The lookup table for kaiser2 does not exist, run');
      error('kaiser2(''setup'') to build the table');
   end
   load('kaiser2.mat')
	B = interp1(NullFreq,SideLobePow,arg1,'linear'); 
otherwise
   error(['Unknown first input parameter: (' N ')']);
end

else
   if ~exist('kaiser2.mat','file')
      disp('The lookup table for kaiser2 does not exist, run');
      error('kaiser2(''setup'') to build the table');
   end
   if nargin < 2
      error('Kaiser2 needs at least 2 inputs, when not running setp')
   end
   
   %This loads 3 variables, alpha, NullFreq, and SideLobePow
   load('kaiser2.mat')
   
   %Now find the index in the lookup table which best matches the
   %window length. Assume window length of table is done
   %in powers of two.
   %index = min(round(log2(N)),round(log2(max(WindLength))))-min(log2(WindLength))+1;
     index = 1;	%I changed the look up to only a single table
   if isstr(arg1)
      %Using the Transition width to find alpha
      if nargin < 3
         error('Kaiser2 must have 3 inputs for transition width calulation')
      end
      if ((arg2<1)|(arg2>11.2))
         error('The transition width much be in the range 1 to 11.2');
      end
      a = interp1(NullFreq,alpha,arg2,'linear');	
   else
      if ((arg1>-13.3)|(arg1<-280))
         error('Sidelobe Power range is -13.3 to -280 dBc');
      end
      
      a = interp1(SideLobePow(:,index),alpha,arg1,'cubic');
   end
   B = kaiser(N,a);
end
