function s0041_ofdmsim(SimFile)
%This simulates OFDM and measures its the performance as a function of AWGN.
%This is a general purpose simulator. Each simulation is stored in a simulation
%file which is generated by newsim. This is a restartable simulation, i.e. it can
%be cancelled and restarted. This is useful if the computer crashes. After each
%EBNR simulation step, it saves the state of the simulation.
%
% Copyright Eric Lawrey March 2001

if nargin <1
   error('You must specify a simulation file to simulate, this can be generated from newsim');
end
load(SimFile)

%Initialisation
carriers = mkcarriers(Ncarriers,IFFTsize,RealComplex);

if strcmp(RealComplex,'complex')
   [delpro] = gensimpimp(Ntaps,delspread,'complex',dellist,decay);
else
   [delpro] = gensimpimp(Ntaps,delspread,dellist,decay);
end
tic							%Start the timer
EBNRdB = sort(EBNRdB);	%Make the list in ascending order
initdisp
getobjhandles
plotresults
NbitsList = round(log2(Nconstellation(ModNumberList)));
while (NumRuns<=MaxRuns)
   for m = ModIndex:length(ModNumberList)
      ModIndex = m;		%Update ModIndex so that if the simulation is stopped it will
      %						 restart from the lastest modulation
      ModNumber = ModNumberList(m);
      SER = zeros(size(EBNRdB))';
      BER = SER';
      if length(MaxTests)>1
	      %Get the maximum simulation tests for the current modulation scheme
         MaxTests2 = MaxTests(m);
      else
         MaxTests2 = MaxTests;
      end
      %Get the lookup table for the modulation scheme, This is constant over all tests.
      [M,Nbits] = data2iqmap(char(ModType(ModNumber)),DiffFlag);  
    
      %Run the simulation until the required number of errors has been exceeded (BitsReq)
      %Or that the number of trials has been exceeded (MaxTests), where each trial tests
      %Nwords.
      for k = EBNRIndex:length(EBNRdB)
         EBNRIndex = k;	%Keep Track of the EBNR stage of simulation
         runbiterrors = 0;
         EBNR = 10.^(EBNRdB(k)/10);
         for l = 1:MaxTests2
            
            if biterrors(k,m) == 0
               %If there were not error found check to see if is the neighbouring
               % lower EBNR simulations had errors. This will indicate whether
               % we are simulation near the cut off BER (1e-5,1e-6) or at a
               % EBNR where we will never get any errors.
               if (NwordsUsed(k,m)*Nbits > BitSimThreshold)
                  %If we have simulated enough bits to reach the BitSimThreshold then
                  %if the current and previous ENBR simulations have a BER of 0
                  %then don't bother further simulating this ENBR, we would be just wasting
                  %CPU cycles.
                  if (k>PrevENBRcheck)
                     pasttwoerrors = sum(biterrors((k-PrevENBRcheck):k,m));
                     if pasttwoerrors == 0
                        breakflag = 1;
                        break
                     end
                  end
               end
            end
            %For times when the BER is low give an update of the simulation
            if (rem(l,DispRate)==0)
               if SNRtest, sSNR = 'SNR'; else, sSNR = 'EBNR'; end
               if OFDMflag, sOFDM = 'OFDM'; else, sOFDM = 'base band'; end
               if DiffFlag
                  ModString = ['D' char(ModType(ModNumber))];
               else
                  ModString = char(ModType(ModNumber));
               end
               dispresults
               drawnow
            end
            
            %====================================
            %			GENERATE DATA
            %====================================
            %Generate random data for the simulation. The data is in columns, with one
            %symbol per column.
            if DiffFlag
               NumSymb = ceil(Nwords/Ncarriers)+1; %Have one extra word for phase reference
               SimNwords=(NumSymb-1)*Ncarriers;
            else
               NumSymb = ceil(Nwords/Ncarriers); %Add a pilot symbol to the start
               SimNwords=NumSymb*Ncarriers;
            end
            %For coherent modulation the the first symbol is a pilot symbol, where
            %the data sent is known by the receiver. For differential modulation the
            %first symbol is a phase reference
            Din = floor(rand(Ncarriers,NumSymb)*Nconstellation(ModNumber));
            
            if DiffFlag
               %====================================================================================
               %								DIFFERENTIAL MODULATION	(IQ Mapping)
               %====================================================================================
               switch ModCatergory(ModNumber)
               case 1
                  %====================================
                  %			Differential QAM square
                  %====================================                  
                  %Convert the data to pairs of words, where each word is half the number of bits
                  %Each half word corresponds to the I and Q mapping. This will be differentially
                  %encoded. This only works for regularly spaced square shaped modulation schemes.
                  %The first word become the phase reference.
                  %Differential QAM does not have any real advantage over coherent QAM.
                  
                  Idata = zeros(size(Din));
                  Qdata = zeros(size(Din));
                  for sb = 1:NumSymb
                     D = convbase(Din(:,sb),Nbits,Nbits/2);
                     Idata(:,sb) = D(:,1);
                     Qdata(:,sb) = D(:,2);
                  end
                  
                  %Idata, Qdata each column is a symbol.
                  %Differentially map the I and Q data
                  ID = rem(cumsum(Idata')',2^(Nbits/2));
                  QD = rem(cumsum(Qdata')',2^(Nbits/2));
                  %Now recombine the data into a single data words ready to convert
                  %to IQ modulation
                  
                  DD2 = zeros(size(Din));
                  for sb = 1:NumSymb
                     D2 = [ID(:,sb) QD(:,sb)];
                     DD2(:,sb) = convbase(D2,Nbits/2,Nbits)';
                  end
                  
               case 0
                  %=====================================
                  %			Differential PSK
                  %=====================================
                  DD2 = rem(cumsum(Din')',2^(Nbits));
                  
               case 2
                  %Differential QAM only works for even number of bits
                  disp(['Error: Word size must be even for differential encoding'])
                  error([char(ModType(ModNumber)) ' can not be used'])                  
               otherwise
                  error('Illegal modulation category')
               end
               %Use the modulation lookup table 'M' to from the IQ vector for each data word in DD2
               IQ = M(DD2+1); 
               %Remove the first data word as it is the phase reference
               Din = Din(:,2:end);
            else               
               %====================================================================================
               %								COHERENT MODULATION (IQ Mapping)
               %====================================================================================            
               %Map the data to complex I/Q using the lookup table
               IQ = M(Din+1);
               %Add pilot symbols to the start of the transmission
               RefSymb = genref(Ncarriers,RefScheme);
               IQ = [repmat(RefSymb,1,NumRefSymb) IQ];
            end
            
            if OFDMflag
               %=============================================
               %		Generate transmitted OFDM signal
               %=============================================
               outsymbol = ofdmmod(IQ, carriers, IFFTsize, GuardPeriod, ...
                  RealComplex, fast_mode);
               
               %If the modulation is real, then the effective bandwidth of the 
               %OFDM spectrum is 2*the number of carriers.
               %This is done to calculate the correct amount of noise to add
               if strcmp(RealComplex,'real')
                  BWscale = 2*Ncarriers/IFFTsize;
               else
                  BWscale = Ncarriers/IFFTsize;
               end
               if SNRtest == 0
                  BWscale = BWscale*Nbits;	%This scaled the noise energy so that the 
                  %							 		energy per bit is correct
               end
               %Convert from a matrix form of one symbol per column to 
               %a single time vector
               outsig = reshape(outsymbol,1,size(outsymbol,1)*size(outsymbol,2));
               
               %======================================
               %		Add AWGN to the signal
               %======================================
               [outsigrx noise] = addnoise(outsig,EBNRdB(k),BWscale);
               
               % outsigrx = [0 outsigrx(1:end-1)];	%Add a small time error
               
               %Change the time domain data into matrix form, one symbol per column.
               outsymbolrx = reshape(outsigrx,(IFFTsize+GuardPeriod),length(outsigrx)/(IFFTsize+GuardPeriod));
               
               %==========================================               
               %		Demodulate the OFDM signal
               %==========================================
               %Strip off the guardperiod
               %This will be the lower rows of the matrix. Each column is a symbol
               %This simulation assumes perfect time synchronization
               NoGuard = outsymbolrx(GuardPeriod+(1:IFFTsize),:);
               %Apply the FFT
               fftdata = fft(NoGuard);
               %Now keep the carriers which were used
               IQrx = fftdata(carriers,:);          
            else
               %=========================================
               %		Base Band Simulation
               %=========================================
               if SNRtest == 0
                  BWscale = Nbits;	%This scaled the noise energy so that the 
                  %						 energy per bit is correct
               else
                  BWscale = 1;
               end
               %Add noise to the waveform
               IQrx = addnoise(IQ,EBNRdB(k),BWscale);
            end
            
            %==========================================
            %		Demodulate the Rx IQ vectors
            %==========================================
            if DiffFlag
               switch ModCatergory(ModNumber)
               case 1
                  %====================================
                  %			Differential QAM square
                  %====================================    
                  %This algorithm does NOT use iq2data
                  %Take the difference between the IQ symbols             
                  IQdiff = diff(IQrx')';
                  %Normalise to the grid step size
                  GridStep = real(M(2)-M(1));
                  
                  %This demodulation assumes a square modulation
                  IQs = round(IQdiff/GridStep);
                  R = rem(real(IQs),2^(Nbits/2));
                  I = rem(imag(IQs),2^(Nbits/2));
                  IQnorm = R+i*I;
                  ind = find(real(IQnorm)<0);
                  IQnorm(ind) = IQnorm(ind)+ones(size(ind))*(2^(Nbits/2));
                  ind = find(imag(IQnorm)<0);
                  IQnorm(ind) = IQnorm(ind)+i*ones(size(ind))*(2^(Nbits/2));
                  
                  %Complex part is the most significant nibbles
                  Do = zeros(size(IQnorm));
                  for sb = 1:size(IQnorm,2)
                     D2 = [real(IQnorm(:,sb)) imag(IQnorm(:,sb))];
                     Do(:,sb) = convbase(D2,Nbits/2,Nbits)';
                  end               
               case 0
                  %====================================
                  %			Differential PSK
                  %====================================
                  %Calculating the data using angles removes the need for knowing the
                  %amplitude scaling
                  %DiffPh = diff(angle(IQrx)')';
                  DiffPh = angle(IQrx(:,2:end)./IQrx(:,1:end-1));
                  
                  %Make sure DiffPh is between -2*pi and +2*pi
                  DiffPh = rem(DiffPh,2*pi);
                  %Make all the phase angles positive
                  ind = find(DiffPh<0);
                  DiffPh(ind) = DiffPh(ind)+2*pi;
                  
                  %ind = find(DiffPh>pi);
                  %DiffPh(ind) = DiffPh(ind)-2*pi;
                  %ind = find(DiffPh<-pi);
                  %DiffPh(ind) = DiffPh(ind)+2*pi;
                  
                  %Do = (abs(DiffPh)>pi/2);
                  %Quantise the phase to get the data out. This round could end up
                  %with data which has wrapped, i.e. for QPSK, 5 deg = 0, 355 deg = 4
                  %which actually is a zero
                  Do = round(DiffPh/(2*pi).*Nconstellation(ModNumber));
                  Ind = find(Do == Nconstellation(ModNumber));
                  Do(Ind) = zeros(size(Ind));
               end
               
            else   
               %=================================
               %		Demodulate Coherent
               %=================================
               %Use the pilot symbol to compensate channel rotations, caused
               %by time offset errors.
               if NumRefSymb == 0
                  %If there is no reference symbols, don't perform channel
                  %equalisation
                  IQrx2 = IQrx;
               else
                  
                  if NumRefSymb > 1
                     ChannResp = (mean(IQrx(:,1:NumRefSymb)')')./(mean(IQ(:,1:NumRefSymb)')');
                  else
                     ChannResp = (IQrx(:,1)./IQ(:,1));
                  end
                  
                  %figure(2)
                  %clf
                  %subplot(2,1,1)
                  %plot(abs(ChannResp))
                  %subplot(2,1,2)
                  %plot(angle(ChannResp));
                  %pause
                  IQrx2 = IQrx(:,NumRefSymb+1:end)./repmat(ChannResp,1,size(IQrx,2)-NumRefSymb);
               end
               %Demodulate the I/Q data, using an optimal decoder. (It finds the
               %distance to each allowable IQ data location then picks the one with the
               %minimum distance). This is effective but can be slow for modulation
               %schemes with large numbers of IQ locations such as 4096QAM
               Do = iq2data(IQrx2,char(ModType(ModNumber)));
            end
            
            %======================================
            %			Record Errors
            %======================================
            %Keep record of number of bits sinulated            
            NwordsUsed(k,m) = NwordsUsed(k,m)+length(Do(:));
            %Find symbol errors and add them to the total
            errorstotal(k,m) = errorstotal(k,m)+length(find(abs(Do-Din)>0));
            %Measure bit errors, by comparing the binary in and out
            Bi = convbase(Din,Nbits,1);	%Convert the input data to bits
            Bo = convbase(Do,Nbits,1);		%Convert the output data to bits 
            biterrors(k,m) = biterrors(k,m) + sum(sum(abs(Bi-Bo)));
            runbiterrors = runbiterrors+sum(sum(abs(Bi-Bo)));
            
            %If sufficient errors have been detected break the loop
            if runbiterrors > BitsReq
               breakflag = 1;
               break
            end
         end
         
         
         %Calculate the SER and BER based on the accumulated error and the number of words
         %simulated
         warning off
         SER = errorstotal(:,m)./NwordsUsed(:,m);
         BER = biterrors(:,m)./(NwordsUsed(:,m)*Nbits);
         warning on
         %Keep a record of the total number of bits simulated. This is just
         %for benchmarking purposes.
         totbitsim = sum(sum(NwordsUsed.*repmat(NbitsList,size(NwordsUsed,1),1)));
         BERall(:,m) = BER;
         SERall(:,m) = SER;
         TotalTime = TotalTime+toc;
         tic;
         save(SimFile, 'ModType','Nconstellation','ModCatergory','Nwords','MaxTests','BitsReq','DispRate',...
            'OFDMflag','Ncarriers','IFFTsize','GuardPeriod','RealComplex','fast_mode','DiffFlag',...
            'SNRtest','EBNRdB','ModNumberList','errorstotal','NwordsUsed','biterrors','BERall',...
            'SERall','totbitsim','Ntaps','delspread','dellist','decay','limits','MaxRuns','NumRuns',...
            'ModIndex','EBNRIndex','ModsUsed','TotalTime','InLineChanChar','NumRefSymb','RefScheme',...
            'BitSimThreshold','PrevENBRcheck');
         
      end
      EBNRIndex = 1;
      plotresults
   end
   NumRuns = NumRuns+1;
   ModIndex = 1;
end


