function [xout,yout,zout] = rtpathloss(txloc,filename,modeltype,range,horzsize,vertsize,coloraxis,gridscale,...
   frequency,pathlossexp)
%RTPATHLOSS raytraces a model to produce shadowing
%[xout,yout,zout] = rtpathloss(txloc,filename,modeltype,range,horzsize,vertsize,coloraxis,gridscale,...
%   frequency,pathlossexp)

%Copyright Eric Lawrey August 2000
%Version 1.1
%4/8/00 Version 1.1
%-Changed the meshing so that less points are recorded near the tx location. This is to
%hopefully improved the triangulation meshing used in griddata.
%-Added default values for all input signals
%4/8/00 Version 1.2
%-Rtpathloss now checks the triangulation of the polar mesh before raytracing. It finds a
%grid spacing which works before doing the raytracing. The raytracing section was more vectorised
%improving the render speed. The total speed improvements are 30-100%
%5/8/00 Version 1.2a
%-Displays a warning that a bad raytrace has been done, by checking if the pathloss is greater
%than 300 dB. No fix is performed though.
%-Removed pause command if the maximum number of grid trials is exceeded.
%10/11/00 Version 1.2b
%-disabled warnings around the grid data command, as to comes up with colinear point warnings
if nargin < 1, txloc = 20.15+i*10.1; end
if nargin < 2, filename = 'pathdefault'; end
if nargin < 3, modeltype = 1; end
if nargin < 4, range = [-10 65 -10 25]; end
if nargin < 5, horzsize = 640; end
if nargin < 6, vertsize = 480; end
if nargin < 7, coloraxis = [-130 -30]; end
if nargin < 8, gridscale = 3; end
if nargin < 9, frequency = 990e6; end
if nargin < 10, pathlossexp = 2.6; end

objs = genmodel(modeltype); 		%Get the ray trace model to use
Nrays = round(400/gridscale);		%Number of rays to use
minx = range(1);
miny = range(3);
maxx = range(2);
maxy = range(4);
boundaries = [minx+i*miny minx+i*maxy maxx+i*maxy maxx+i*miny];
radius = max(abs(txloc-boundaries))*1.1;
theta = linspace(-pi,pi-(2*pi/Nrays),Nrays);
z = radius * exp(i*theta)+txloc;
height = 2;						%Transmitter height. This is the minimum distance to the
%									transmitter even at the txloc
GridSize = 0.3*gridscale;	%Grid size in metres
NaNflag = 1;
MaxRuns = 12;					%Maximum number of retries to get a data set with no NaN data

xlin = minx:GridSize:maxx;
ylin = miny:GridSize:maxy;
[X,Y] = meshgrid(xlin,ylin);
Zf = ones(size(X))*NaN;
PolarGridSize = 0.6*gridscale;
LoopCounter = 0;
wavelen = 3e8/frequency;	%wave length
tic
%Minimum distance to keep ray trace data.
rmin = 0.5;
while NaNflag == 1
   PolarGridSize = PolarGridSize+0.15*gridscale;
   %Nrays = round(Nrays*0.9);
   %Vectorised method for generating the polar coordinates
   %This is about 50 times faster than the for loop method used
   %in the next section
   r = rmin:PolarGridSize:radius; %Distance travelled
   r2 = repmat(r',1,Nrays);
   ph2 = repmat(theta,length(r),1);
   zk = r2 .* exp(i*ph2)+txloc;
   x = real(zk(:))';
   y = imag(zk(:))';

   %Add the transmitter location in as it allows correct triangulation
   x = [x real(txloc)];
   y = [y imag(txloc)];
   %toc
%Z = griddata(x,y,zeros(size(x)),X,Y,'linear');
GridPassFlag = ChkGrid(x,y,X,Y);
%if sum(isnan(Z))==0
if GridPassFlag == 1
      disp('Grid passed')
      NaNflag = 0;
   else
      disp('Grid failed, Checking new grid')
   end
   LoopCounter = LoopCounter+1;
   if LoopCounter > MaxRuns
      NaNflag = 0;
      disp('Grid failed after maximum number of trials. Continuing anyway');
   end
   %toc
end
%disp('Found Good Grid')
z2 = zeros(1,length(x));
r = rmin:PolarGridSize:radius; %Distance travelled
Nr = length(r);
Iz = 1;
for k = 1:length(z)
   %if k == 101
   %   disp('tteest')
   %end
   
   line_ends = [txloc z(k)];
   %Find the intercept points for the current ray
   [intercept_points, obj_surface_angle, cum_atten, collision_object] = rayline2(objs,line_ends);
   unitvect = (z(k)-txloc)/abs(z(k)-txloc);
   intercept2 = [intercept_points z(k)];
   inter_dist = abs(intercept2-txloc);		%Distance to each intercept point
   linepathloss = (wavelen./(4*pi*sqrt(r.^2+height.^2))).^pathlossexp;
   d = zeros(1,Nr); d(1) = rmin;
   za = zeros(1,Nr); za(1) = cum_atten(1);
   Ik = 2;
   for l = 1:length(intercept2)
      dnew = (d(Ik-1)+PolarGridSize):PolarGridSize:inter_dist(l);
      d(Ik:(Ik+length(dnew)-1)) = dnew;
      za(Ik:(Ik+length(dnew)-1)) = ones(1,length(dnew))*cum_atten(l);
      Ik = Ik+length(dnew);
   end
   
   lineloss = 10*log10(linepathloss);
   totalloss = lineloss-za;
   if min(totalloss)<-300
      disp(['hit bad bit : ' num2str(k)]);
   end
   
   %plot(d,totalloss)
   %title(k)
   %pause
   linevect = r*unitvect+txloc;
   %Now record all the vector locations and the corresponding z values into a
   %big array so that it can be used to interpolate back to regular cartesian
   %coordinates
	z2(Iz:Iz+length(totalloss)-1) = totalloss;
	Iz = Iz+length(totalloss);
   if rem(k,40)==0
      disp([num2str(txloc) ', ' int2str(k) ' of '  int2str(Nrays)])
      drawnow
   end
end
%Add the transmitter location in as it allows correct triangulation
x = [x real(txloc)];
y = [y imag(txloc)];
z2 = [z2 0];
clf
warning off
Z = griddata(x,y,z2,X,Y,'linear');
warning on

%Save all grid points which are not NaN values

if sum(isnan(Z))==0
   Zf = Z;
   NaNflag = 0;
else
   Zind = find(~isnan(Z));
   Zf(Zind) = Z(Zind);
   disp('Data had NaN values so repeating calculations with new grid size');
end 
plotresult(xlin,ylin,Zf,objs,filename,horzsize,vertsize,coloraxis)
if nargout > 0 
   xout = xlin;
   yout = ylin;
   zout = Zf;
end
t = toc;
disp(['render time : ' num2str(t) 'sec']);