function s0011_expcompare
%This script simulates the practical measurements done using multiple transmitters.
%It finds the pathloss distribution over the area of the ECE building (2nd floor)

%Copyright Eric Lawrey 6/8/00
%Version 1.1
%6/8/00 version 1.1
%Added plotting of the measured data, and direct comparision with simulated data.
%Also added calculation and plotting of 4 transmitters and 8 transmitters.
clear all

%Load measured data
M = dlmread('measdata.csv',',');
RxGain = 60-45.6;	%Receiver Preamp gain, including all losses in dB		
TxPow = 4.4;		%Transmitter power in dBm, both tx1 and tx2 should be matched
lb = 52.7;		%Length of building
yoff = 0.3;	%yoffset (thickness of outer wall
xscale = 0.3;	%xscale to get x im metres
yscale = 0.295;
XM = lb-M(:,1)*xscale;		%Scale input x data, which is based on tile spacings (0.3m) into metres
YM = M(:,2)*yscale+yoff;		%Scale input y data
Tx1 = -(M(:,3)-RxGain-TxPow);		%Calc pathloss, assume input data is in dBm
Tx2 = -(M(:,4)-RxGain-TxPow);
Nx = 200;
Ny = 60;
xlin = linspace(min(XM),max(XM),Nx);
ylin = linspace(min(YM),max(YM),Ny);
[X2,Y2] = meshgrid(xlin,ylin);
Z1 = griddata(XM,YM,Tx1,X2,Y2,'linear');
Z2 = griddata(XM,YM,Tx2,X2,Y2,'linear');

Caxis = [51  95];
ColorAxis = [-95 -51];
horzsize = 640;
vertsize = 220;
aspect = vertsize/horzsize;
colorbar = 0;							%Turns on the colorbar when plotting the results
minx = 0;
miny = 0;
maxx = 53;
maxy = 16.8;
range = [minx maxx miny maxy];
ContourLines = 60:10:90;							%Contour lines
N = 5; 									%Number of simulations to use to sim diffraction
diff_dist = 0.3; 						%distance between diffraction simultatons in metres
tx1 = [24.6500+10.9150*i]; 		%Position of Txs in experiment
tx2 = [17.4500+10.9150*i  37.8500+10.9150*i]; %Experiment position
xm =  [53.1198 39.0323 28.8940 16.5]-10;
ym =  [36.7193 42.9825 36.6667 42.6316]-30;
tx4 = xm+i*ym;							%Position of the fixed transmitters
xm = [14.48 17.62 26.23 32.50 38.96 43.86 56.98 56.79 ]-10;
ym = [34.14 42.71 34.64 42.92 33.71 43.14 33.92 43.99 ]-30;
tx8 = xm+i*ym;							%Position of the fixed transmitters

modeltype = 1;
gridscale = 4;							%Scaling of the ray trace grid spacing. gridscale = 1, 
%											 400 rays and ~0.5m grid spacing 
frequency = 990e6;
pathlossexp = 2.65;						%Pathloss exponent, 2 = free space loss
Colourmap = max(0,min(1,1.5*((1-pink).^0.5)));	%Colour map to use for the surface plots

objs = genmodel(modeltype); 		%Get the model for the ECE building at JCU
figure(1)
filename = 's0011_1tx_meas';
plotpathloss(X2,Y2,Z1,Colourmap,Caxis,ContourLines,tx1,objs)
savefig(filename,'jpg')
figure(2)
filename = 's0011_2tx_meas';
plotpathloss(X2,Y2,Z2,Colourmap,Caxis,ContourLines,tx2,objs)
savefig(filename,'jpg')

figure(3)
%Raytrace shadows for one transmitter
filename = 's0011_1tx_pathloss';
[x,y,z1] = rtdiffract(tx1,diff_dist,filename,modeltype,range,horzsize,vertsize,ColorAxis,N,...
   gridscale,frequency,pathlossexp);
plotpathloss(x,y,-z1,Colourmap,Caxis,ContourLines,tx1,objs)
savefig(filename,'jpg')

figure(4)
[x,y,z2] = multitx(tx2,diff_dist,modeltype,range,horzsize,vertsize,ColorAxis,N,...
   gridscale,frequency,pathlossexp);
filename = 's0011_2tx_pathloss';
plotpathloss(x,y,-z2,Colourmap,Caxis,ContourLines,tx2,objs)
savefig(filename,'jpg')

figure(5)
[x,y,z4] = multitx(tx4,diff_dist,modeltype,range,horzsize,vertsize,ColorAxis,N,...
   gridscale,frequency,pathlossexp);
filename = 's0011_4tx_pathloss';
plotpathloss(x,y,-z4,Colourmap,Caxis,ContourLines,tx4,objs)
savefig(filename,'jpg')

figure(6)
[x,y,z8] = multitx(tx8,diff_dist,modeltype,range,horzsize,vertsize,ColorAxis,N,...
   gridscale,frequency,pathlossexp);
filename = 's0011_8tx_pathloss';
plotpathloss(x,y,-z8,Colourmap,Caxis,ContourLines,tx8,objs)
savefig(filename,'jpg')



%Only select the area in the building
indx = find((x>1)&(x<51));
indy = find((y>1)&(y<16));
indxm = find((xlin>1)&(xlin<51));
indym = find((ylin>1)&(ylin<16));
zb1 = -z1(indy,indx);
zb2 = -z2(indy,indx);
zb4 = -z4(indy,indx);
zb8 = -z8(indy,indx);
zm1 = Z1(indym,indxm);
zm2 = Z2(indym,indxm);


%Find the pathloss probability distribution
[n1,x1] = hist(zb1(:),50);
nr1 = n1(length(n1):-1:1);
xr1 = x1(length(x1):-1:1);
[n2,x2] = hist(zb2(:),50);
nr2 = n2(length(n2):-1:1);
xr2 = x2(length(x2):-1:1);
[n4,x4] = hist(zb4(:),50);
nr4 = n4(length(n4):-1:1);
xr4 = x4(length(x4):-1:1);
[n8,x8] = hist(zb8(:),50);
nr8 = n8(length(n8):-1:1);
xr8 = x8(length(x8):-1:1);

[nm1,xm1] = hist(zm1(:),50);
nrm1 = nm1(length(nm1):-1:1);
xrm1 = xm1(length(xm1):-1:1);
[nm2,xm2] = hist(zm2(:),50);
nrm2 = nm2(length(nm2):-1:1);
xrm2 = xm2(length(xm2):-1:1);

figure(7)
clf
[h] = semilogy(xr1,cumsum(nr1)/sum(nr1),'b-',xr2,cumsum(nr2)/sum(nr2),'r.-',...
   xr4,cumsum(nr4)/sum(nr4),'gx-',xr8,cumsum(nr8)/sum(nr8),'k^-');
grid on
%[n,x] = hist(-Z2(:),100); [n1,x1] = hist(-Z1(:),100);
%[h]=semilogy(-x,cumsum(n)/sum(n),-x1,cumsum(n1)/sum(n1))
legend(h,'1 Tx','2 Tx','4 Tx','8 Tx');
axis([60 105 0.01 1])
xlabel('Pathloss (dB)');
ylabel('Probability of worse pathloss');
setplotstyle(2,1.1)
legend
set(h(2),'markersize',30);
set(h(3),'markersize',15);
set(h(4),'markersize',12);
legend
savefig('s0011_sim_probdist')

figure(8)
clf
[h] = semilogy(xrm1,cumsum(nrm1)/sum(nrm1),'r-',xrm2,cumsum(nrm2)/sum(nrm2),'r.-',...
   xr1,cumsum(nr1)/sum(nr1),'b-',xr2,cumsum(nr2)/sum(nr2),'b.-');
grid on
%[n,x] = hist(-Z2(:),100); [n1,x1] = hist(-Z1(:),100);
%[h]=semilogy(-x,cumsum(n)/sum(n),-x1,cumsum(n1)/sum(n1))
legend(h,'1 Tx (Meas)','2 Tx (Meas)','1 Tx (Sim)','2 Tx (Sim)');
axis([60 105 0.01 1])
xlabel('Pathloss (dB)');
ylabel('Probability of worse pathloss');
setplotstyle(3,1.1)
legend
set(h(1),'linewidth',1.5);
set(h(2),'linewidth',1.5);
legend
savefig('s0011_meas_probdist')

function plotpathloss(x,y,z,Colourmap,Caxis,ContourLines,tx1,objs)
clf
colourscale = 0.03;
subplot('position',[0.08 0.2 0.85 0.6])
pcolor(x,y,z);
colormap(Colourmap);
caxis([Caxis]);
shading interp
hold on
[c,h] = contour(x,y,z,ContourLines,'k-');
if ~isempty(c)
   clabel(c,h,'fontsize',18);
end
hold off
axis equal

h = colorbar('horiz');
h2 = get(h,'xlabel');
set(h2,'string','Pathloss (dB)');
xlabel('(m)');
ylabel('(m)');
setplotstyle
hold on
plotmodelobj(objs,colourscale);
axis tight
hold off
hold on
plot(tx1,'rx','markersize',25,'linewidth',3);
hold off

function [x,y,zout] = multitx(tx,diff_dist,modeltype,range,horzsize,vertsize,Caxis,N,...
   gridscale,frequency,pathlossexp);
for k = 1:length(tx)
%Raytrace shadows for both of the transmitters, in the two transmitter experiment
[x,y,z] = rtdiffract(tx(k),diff_dist,'default',modeltype,range,horzsize,vertsize,Caxis,N,...
   gridscale,frequency,pathlossexp);
if k == 1
   ztotal = 10.^(z/10);
else
   ztotal = 10.^(z/10)+ztotal;
end
zout = 10*log10(ztotal/length(tx));
end